import 'package:google_applovin_unity_ads/ads/SettingsModel.dart';
import 'package:google_applovin_unity_ads/google_applovin_unity_ads.dart';
import 'package:flutter/widgets.dart';

const AdRequest _request = AdRequest(
  // keywords: <String>['foo', 'bar'],
  // contentUrl: 'http://foo.com/bar.html',
  // nonPersonalizedAds: true,
);

const AdManagerAdRequest _ad_manager_request = AdManagerAdRequest(
  // keywords: <String>['foo', 'bar'],
  // contentUrl: 'http://foo.com/bar.html',
  // nonPersonalizedAds: true,
);


BannerAd? _bannerAd;
NativeAd? _nativeAd;
AdManagerBannerAd? _adManagerBannerAd;

const String _GOOGLEAD = "google_ads";
const String _ADMANAGERAD = "ad_manager";
const String _APPLOVINAD = "app_lovin";
const String _FACEBOOKAD = "facebook_ads";
const String _CUSTOMAD = "custom_ads";

Settings? _settings = getAdsSettings();
int appVersionCode = getAppVersionCode();

void destroyAd() {
  getInterestitialAd()?.dispose();
  getAdManagerInterstitialAd()?.dispose();
  getRewardedAd()?.dispose();
  _adManagerBannerAd?.dispose();
  _bannerAd?.dispose();
  _nativeAd?.dispose();
}

initOpenAds({Function? onOpenAdLoaded, Function? onOpenAdFailedLoaded}) {
  if (_settings == null) {
    if (onOpenAdLoaded != null) {
      onOpenAdLoaded();
    }
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    if (onOpenAdLoaded != null) {
      onOpenAdLoaded();
    }
    return;
  }

  final adUnitId = _settings?.googleAds?.appOpenAdId ?? "";
  AppOpenAdManager appOpenAdManager = AppOpenAdManager()..loadAd(adUnitId: adUnitId, callback: onOpenAdLoaded);
  setAppLifecycleReactor(AppLifecycleReactor(appOpenAdManager: appOpenAdManager, appVersionCode: appVersionCode));
  getAppLifecycleReactor()?.listenToAppStateChanges();
}

showOpenAds() {
  if (_settings == null) {
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    return;
  }

  final adUnitId = _settings?.googleAds?.appOpenAdId ?? "";
  getAppLifecycleReactor()?.appOpenAdManager.showAdIfAvailable(adUnitId);
}

showIntraAds({Function? callback}) async {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    if (callback != null) {
      callback();
    }
    return;
  }

  final isInterestial = ((_settings?.adSetting?.interestialAdsInterval ?? 0) >
      0 &&
      (_settings?.adSetting?.interestialAdsIntervalCount ?? 0) %
          (_settings?.adSetting?.interestialAdsInterval ?? 0) ==
          0 &&
      ((getInterestialAdTimeInterval() +
          ((_settings?.adSetting?.interestialAdsTimingInterval ?? 0) *
              1000)) <=
          DateTime
              .now()
              .millisecondsSinceEpoch ||
          getInterestialAdTimeInterval() == 0));

  _settings?.adSetting?.interestialAdsIntervalCount =
      (_settings?.adSetting?.interestialAdsIntervalCount ?? 0) + 1;
  if (!isInterestial) {
    if (callback != null) {
      callback();
    }
    loadIntraAds();
    return;
  }

  _showInterstitialAd(callback);
}

showRewardAds({Function? callback}) async {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    if (callback != null) {
      callback();
    }
  } else {
    _showRewardedAd(callback);
  }
}

showBannerAds(
    {int pos = 0, AdSize size = AdSize.banner}) {
  if (_settings == null ||
      (_settings?.adSetting?.appVersionCode == appVersionCode) ||
      (_settings?.adsSequence?.isEmpty ?? true) ||
      ((_settings?.adsSequence?.length ?? 0) <= pos)) {
    return Container(
      height: 0,
    );
  }

  switch (_settings?.adsSequence?[pos]) {
    case _GOOGLEAD:
      if (_settings?.googleAds?.isBannerAds ?? false) {
        _bannerAd = BannerAd(
            size: size,
            adUnitId: _settings?.googleAds?.bannerId ?? "",
            listener: BannerAdListener(
              onAdLoaded: (Ad ad) {
                print('$BannerAd loaded.');
              },
              onAdFailedToLoad: (Ad ad, LoadAdError error) {
                print('$BannerAd failedToLoad: $error');
                ad.dispose();
                showBannerAds(pos: pos + 1);
              },
              onAdOpened: (Ad ad) => print('$BannerAd onAdOpened.'),
              onAdClosed: (Ad ad) => print('$BannerAd onAdClosed.'),
            ),
            request: _request)..load();

        return Container(
          width: size.width.toDouble(),
          height: size.height.toDouble(),
          child: AdWidget(ad: _bannerAd!),
        );
      }
      break;
    case _ADMANAGERAD:
      if (_settings?.googleAds?.isBannerAds ?? false) {
        _adManagerBannerAd = AdManagerBannerAd(
          adUnitId: _settings?.googleAds?.bannerId ?? "",
          sizes: <AdSize>[size],
          request: _ad_manager_request,
          listener: AdManagerBannerAdListener(
            onAdLoaded: (Ad ad) async {
              print('Inline adaptive banner loaded: ${ad.responseInfo}');

              // After the ad is loaded, get the platform ad size and use it to
              // update the height of the container. This is necessary because the
              // height can change after the ad is loaded.

              /* AdManagerBannerAd bannerAd = (ad as AdManagerBannerAd);
              final AdSize? size = await bannerAd.getPlatformAdSize();

              if (size == null) {
                print('Error: getPlatformAdSize() returned null for $bannerAd');
                return;
              }*/
            },
            onAdFailedToLoad: (Ad ad, LoadAdError error) {
              print('Inline adaptive banner failedToLoad: $error');
              ad.dispose();
              showBannerAds(pos: pos + 1);
            },
          ),
        )..load();
        return Container(
          width: size.width.toDouble(),
          height: size.height.toDouble(),
          child: AdWidget(ad: _adManagerBannerAd!),
        );
      }
      break;
  }

  return Container(
    height: 0,
  );
}

loadNativeAds(
    {int pos = 0, Function? onAdLoadedCallback, Function? onAdFailedToLoadCallback}) async {
  if (_settings == null ||
      (_settings?.adSetting?.appVersionCode == appVersionCode) ||
      (_settings?.adsSequence?.isEmpty ?? true) ||
      ((_settings?.adsSequence?.length ?? 0) <= pos)) {
    if (onAdFailedToLoadCallback != null) {
      onAdFailedToLoadCallback();
    }
    
  }



  switch (_settings!.adsSequence![pos]) {
    case _GOOGLEAD:
      if (_settings?.googleAds?.isNativeAds ?? false) {
        _nativeAd = NativeAd(
          adUnitId: _settings?.googleAds?.nativeId ?? "",
          request: _request,
          factoryId: 'nativeFactoryId',
          listener: NativeAdListener(
            onAdLoaded: (Ad ad) {
              print('$NativeAd loaded.');
              if (onAdLoadedCallback != null) {
                onAdLoadedCallback();
              }
            },
            onAdFailedToLoad: (Ad ad, LoadAdError error) {
              print('$NativeAd == = = ==== ==== ==== ==== ==== ==== === failedToLoad: $error');
              ad.dispose();
              loadNativeAds(pos: pos + 1, onAdLoadedCallback: onAdLoadedCallback, onAdFailedToLoadCallback: onAdFailedToLoadCallback);
            },
            onAdOpened: (Ad ad) => print('$NativeAd onAdOpened.'),
            onAdClosed: (Ad ad) => print('$NativeAd onAdClosed.'),
          ),
        )..load();
        return;
      }
      break;
    case _ADMANAGERAD:
      if (_settings?.googleAds?.isNativeAds ?? false) {
        _nativeAd = NativeAd.fromAdManagerRequest(
            adUnitId: _settings?.googleAds?.nativeId ?? "",
            factoryId: 'nativeFactoryId',
            listener: NativeAdListener(
              onAdLoaded: (Ad ad) {
                print('$NativeAd loaded.');
                if (onAdLoadedCallback != null) {
                  onAdLoadedCallback();
                }
              },
              onAdFailedToLoad: (Ad ad, LoadAdError error) {
                print('$NativeAd failedToLoad: $error');
                ad.dispose();
                loadNativeAds(pos: pos + 1, onAdLoadedCallback: onAdLoadedCallback, onAdFailedToLoadCallback: onAdFailedToLoadCallback);
              },
              onAdOpened: (Ad ad) => print('$NativeAd onAdOpened.'),
              onAdClosed: (Ad ad) => print('$NativeAd onAdClosed.'),
            ),
            adManagerRequest: _ad_manager_request)..load();
        return;
      }
      break;
  }

  if (onAdFailedToLoadCallback != null) {
    onAdFailedToLoadCallback();
  }
}

showNativeAds({Size size = const Size(double.infinity, 350)}) {
  if (_nativeAd != null) {
    return SizedBox(
      width: size.width.toDouble(),
      height: size.height.toDouble(),
      child: AdWidget(ad: _nativeAd!),
    );
  }
  return Container(height: 0);
}

IsFullIntraAds({Function? callback}) {
  if (_settings?.adSetting?.isFullAds ?? false) {
    showIntraAds(callback: callback);
    return;
  }

  if (callback != null) {
    callback();
  }
}

showFullNativeAds({Size? size}) {
  if (_settings?.adSetting?.isFullAds ?? false) {
    return (size != null) ? showNativeAds(size: size) : showNativeAds();
  }
  return Container(height: 0);
}

void loadIntraAds({int pos = 0}) {
  if (_settings == null) {
    return;
  }

  if (_settings?.adsSequence?.isEmpty ?? true) {
    return;
  }

  if ((_settings?.adsSequence?.length ?? 0) <= pos) {
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    return;
  }

  final isInterestial = ((_settings?.adSetting?.interestialAdsInterval ?? 0) >
      0 &&
      (_settings?.adSetting?.interestialAdsIntervalCount ?? 0) %
          (_settings?.adSetting?.interestialAdsInterval ?? 0) ==
          0 &&
      ((getInterestialAdTimeInterval() +
          ((_settings?.adSetting?.interestialAdsTimingInterval ?? 0) *
              1000)) <=
          DateTime
              .now()
              .millisecondsSinceEpoch ||
          getInterestialAdTimeInterval() == 0));

  switch (_settings?.adsSequence?[pos]) {
    case _GOOGLEAD:
    case _ADMANAGERAD:
      if (((_settings?.googleAds?.interstitialInterval ?? 0) <= 0 &&
          isInterestial) ||
          ((_settings?.googleAds?.interstitialInterval ?? 0) > 0 &&
              ((_settings?.adSetting?.interestialAdsIntervalCount ?? 0) %
                  (_settings?.googleAds?.interstitialInterval ?? 0) ==
                  0))) {
        if (_settings?.googleAds?.isInterstitialAds ?? true) {
          if (_ADMANAGERAD == _settings?.adsSequence?[pos]) {
            _createAdManagerInterstitialAd(pos);
            return;
          }
          _createInterstitialAd(pos);
          return;
        }
      }
      break;
  }

  loadIntraAds(pos: pos + 1);
}

void loadRewardAds({int pos = 0}) {
  if (_settings == null) {
    return;
  }

  if (_settings?.adsSequence?.isEmpty ?? true) {
    return;
  }

  if ((_settings?.adsSequence?.length ?? 0) <= pos) {
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    return;
  }

  switch (_settings?.adsSequence?[pos]) {
    case _GOOGLEAD:
    case _ADMANAGERAD:
      if (_settings?.googleAds?.isRewaredAds ?? true) {
        if (_ADMANAGERAD == _settings?.adsSequence?[pos]) {
          _createAdmanagerRewardedAd(pos);
          return;
        }
        _createRewardedAd(pos);
        return;
      }
      break;
  }

  loadRewardAds(pos: pos + 1);
}

void _createInterstitialAd(int pos) {
  if (getInterestitialAd() != null) return;
  InterstitialAd.load(
      adUnitId: _settings?.googleAds?.interestialId ?? "",
      request: _request,
      adLoadCallback: InterstitialAdLoadCallback(
        onAdLoaded: (InterstitialAd ad) {
          print('$ad loaded');
          setInterestitialAd(ad);
          getInterestitialAd()?.setImmersiveMode(true);
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('InterstitialAd failed to load: $error.');
          setInterestitialAd(null);
          loadIntraAds(pos: pos + 1);
        },
      ));
}

void _createRewardedAd(int pos) {
  if (getRewardedAd() != null) return;
  RewardedAd.load(
      adUnitId: _settings?.googleAds?.rewardedId ?? "",
      request: _request,
      rewardedAdLoadCallback: RewardedAdLoadCallback(
        onAdLoaded: (RewardedAd ad) {
          print('$ad loaded.');
          setRewardedAd(ad);
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('RewardedAd failed to load: $error');
          setRewardedAd(null);
          loadRewardAds(pos: pos + 1);
        },
      ));
}

void _createAdmanagerRewardedAd(int pos) {
  if (getRewardedAd() != null) return;
  RewardedAd.loadWithAdManagerAdRequest(
      adUnitId: _settings?.googleAds?.rewardedId ?? "",
      adManagerRequest: _ad_manager_request,
      rewardedAdLoadCallback: RewardedAdLoadCallback(
        onAdLoaded: (RewardedAd ad) {
          print('$ad loaded.');
          setRewardedAd(ad);
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('RewardedAd failed to load: $error');
          setRewardedAd(null);
          loadRewardAds(pos: pos + 1);
        },
      ));
}

void _createAdManagerInterstitialAd(int pos) {
  if (getAdManagerInterstitialAd() != null) return;

  AdManagerInterstitialAd.load(
      adUnitId: _settings?.googleAds?.interestialId ?? "",
      request: _ad_manager_request,
      adLoadCallback: AdManagerInterstitialAdLoadCallback(
          onAdLoaded: (AdManagerInterstitialAd ad) {
            print('$ad loaded');
            setAdManagerInterstitialAd(ad);
            getAdManagerInterstitialAd()?.setImmersiveMode(true);
          },
          onAdFailedToLoad: (LoadAdError error) {
            print('InterstitialAd failed to load: $error.');
            setAdManagerInterstitialAd(null);
            loadIntraAds(pos: pos + 1);
          }
      ));
}

void _showInterstitialAd(Function? callback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getInterestitialAd() == null) {
    _showAdManagerInterstitialAd(callback);
    return;
  }

  getInterestitialAd()?.fullScreenContentCallback = FullScreenContentCallback(
    onAdShowedFullScreenContent: (InterstitialAd ad) =>
        print('ad onAdShowedFullScreenContent.'),
    onAdDismissedFullScreenContent: (InterstitialAd ad) {
      print('$ad onAdDismissedFullScreenContent.');
      ad.dispose();
      setInterestialAdTimeInterval(DateTime
          .now()
          .millisecondsSinceEpoch);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
    onAdFailedToShowFullScreenContent: (InterstitialAd ad, AdError error) {
      print('$ad onAdFailedToShowFullScreenContent: $error');
      ad.dispose();
      // _loadInterstitialAd(pos: pos + 1);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
  );
  getInterestitialAd()?.show();
  setInterestitialAd(null);
}

void _showAdManagerInterstitialAd(Function? callback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getAdManagerInterstitialAd() == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  getAdManagerInterstitialAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
        onAdShowedFullScreenContent: (AdManagerInterstitialAd ad) =>
            print('ad onAdShowedFullScreenContent.'),
        onAdDismissedFullScreenContent: (AdManagerInterstitialAd ad) {
          print('$ad onAdDismissedFullScreenContent.');
          ad.dispose();
          setInterestialAdTimeInterval(DateTime
              .now()
              .millisecondsSinceEpoch);

          if (callback != null) {
            callback();
          }
          loadIntraAds();
        },
        onAdFailedToShowFullScreenContent: (AdManagerInterstitialAd ad,
            AdError error) {
          print('$ad onAdFailedToShowFullScreenContent: $error');
          ad.dispose();
          // _loadInterstitialAd(pos: pos + 1);
          if (callback != null) {
            callback();
          }
          loadIntraAds();
        },
      );
  getAdManagerInterstitialAd()?.show();
  setAdManagerInterstitialAd(null);
}

void _showRewardedAd(Function? callback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getRewardedAd() == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  getRewardedAd()?.fullScreenContentCallback = FullScreenContentCallback(
    onAdShowedFullScreenContent: (RewardedAd ad) =>
        print('ad onAdShowedFullScreenContent.'),
    onAdDismissedFullScreenContent: (RewardedAd ad) {
      print('$ad onAdDismissedFullScreenContent.');
      ad.dispose();
      loadRewardAds();
    },
    onAdFailedToShowFullScreenContent: (RewardedAd ad, AdError error) {
      print('$ad onAdFailedToShowFullScreenContent: $error');
      ad.dispose();
      if (callback != null) {
        callback();
      }
      loadRewardAds();
    },
  );

  getRewardedAd()?.setImmersiveMode(true);
  getRewardedAd()?.show(
      onUserEarnedReward: (AdWithoutView ad, RewardItem reward) {
        print('$ad with reward $RewardItem(${reward.amount}, ${reward.type})');
        if (callback != null) {
          callback();
        }
      });
  setRewardedAd(null);
}